'''
Created on Aug 26, 2009

@author: marumjr
'''

#!/usr/bin/env python

from StandardComponentFactory import *
from ButtonFactory import *
from MenuFactory import *
from ToolFactory import *

class PanelFactory():
    '''Panel Factory class, from which we extract some standard panels'''
    
    ################ MAIN WINDOW PANELS #####################
    
    @staticmethod
    def __draw_black_background(screen, event, m, n):
        '''Set a black background on the drawing screen
        @param screen: gtk.DrawingArea
        @param event: an event
        @param m: amount of drawing areas on the horizontal axis
        @param n: amount of drawing areas on the vertical axis'''
        width, heigth = screen.get_toplevel().get_size()
        width *= (1.0 / m)
        heigth *= (1.0 / n)
        gc = screen.get_style().fg_gc[gtk.STATE_NORMAL]
        black = gtk.gdk.Color(0, 0, 0, 0)
        gc.set_rgb_fg_color(black)
        screen.window.set_background(black)
    
    @staticmethod
    def create_menu_panel(widgets_callback_dictionary, accel_group):
        '''Create the menu panel
        @param widgets_callback_dictionary: a dictionary on which we'll insert the items that need a callback
        @param accel_group: gtk.AccelGroup
        @return: gtk.HBox'''
        menu_box = gtk.HBox(True)
        menu_bar = gtk.MenuBar()
        # The "File" menu
        file_menu = MenuFactory.create_file_menu(widgets_callback_dictionary, accel_group)
        menu_bar.append(file_menu)
        # The "About" menu
        about_menu = MenuFactory.create_help_menu(widgets_callback_dictionary, accel_group)
        menu_bar.append(about_menu)
        menu_box.pack_start(menu_bar, False)
        return menu_box
    
    @staticmethod
    def create_toolbar_panel(widgets_callback_dictionary):
        '''Create the toolbar panel
        @param widgets_callback_dictionary: a dictionary on which we'll insert the items that need a callback
        @return: gtk.HBox'''
        toolbar_box = gtk.HBox(True)
        toolbar = gtk.Toolbar()
        toolbar.set_orientation(gtk.ORIENTATION_HORIZONTAL)
        # Control the number of videos that can be displayed horizontally
        x_spin_button = ToolFactory.create_video_x_display_control()
        widgets_callback_dictionary[x_spin_button.get_name()] = x_spin_button
        toolbar.insert(x_spin_button, -1)
        # Control the number of videos that can be displayed vertically
        y_spin_button = ToolFactory.create_video_y_display_control()
        widgets_callback_dictionary[y_spin_button.get_name()] = y_spin_button
        toolbar.insert(y_spin_button, -1)
        # Separator
        toolbar.insert(gtk.SeparatorToolItem(), -1)
        # Lock Button
        lock_toggletoolbutton = ToolFactory.create_lock_togglebutton()
        widgets_callback_dictionary[lock_toggletoolbutton.get_name()] = lock_toggletoolbutton
        toolbar.insert(lock_toggletoolbutton, -1)
        toolbar_box.pack_start(toolbar, False)
        return toolbar_box
    
    @staticmethod
    def create_filechooser_panel(widgets_callback_dictionary):#, m=1, n=1):
        '''Create the file insertion panel, on which we insert the files to be displayed
        @param widgets_callback_dictionary: a dictionary containing widgets to be connected to a callback
        @param m: number of videos to be displayed, horizontally
        @param n: number of videos to be displayed, vertically
        @return: gtk.HBox'''
        file_insertion_box = gtk.HBox(True)
        table = gtk.Table(1, 1, True)
        file_insertion_box.pack_start(table, True, True, 0)
        widgets_callback_dictionary['Callback_Panel_Filechooser'] = file_insertion_box
        return file_insertion_box
    
    @staticmethod
    def create_flux_control_panel(widgets_callback_dictionary):
        '''Create the flux control panel, on which we can choose what the next step
        @param widgets_callback_dictionary: a dictionary containing widgets to be connected to a callback
        @return: gtk.HBox'''
        outer_box = gtk.HBox(True)
        # Empty space
        left_box = gtk.HBox(True)
        outer_box.pack_start(left_box, True, True)
        # The panel itself
        flux_control_box = gtk.HBox(True)
        view_window_button = ButtonFactory.create_view_window_button()
        view_window_button.set_size_request(-1, 40)
        widgets_callback_dictionary[view_window_button.get_name()] = view_window_button
        flux_control_box.pack_start(view_window_button, True, True)
        outer_box.pack_start(flux_control_box, True)
        # Empty space
        right_box = gtk.HBox(True)
        outer_box.pack_start(right_box, True, True)
        widgets_callback_dictionary['Callback_Panel_Fluxcontrol'] = outer_box
        return outer_box
    
    ################ VIEW WINDOW PANELS #####################
    
    @staticmethod
    def create_video_panel(drawing_areas_list, lines=1, columns=1):
        '''Create a video panel, on which m * n videos are displayed
        @param lines: number of videos to be displayed, vertically
        @param columns: number of videos to be displayed, horizontally
        @return: gtk.HBox'''
        video_box = gtk.HBox(True)
        table = gtk.Table(lines, columns)
        for i in range(columns):
            for j in range(lines):
                frame = gtk.Frame()
                drawing_area = gtk.DrawingArea()
                drawing_areas_list.append(drawing_area)
                frame.add(drawing_area)
                table.attach(frame, i, i + 1, j, j + 1, gtk.EXPAND | gtk.FILL, gtk.EXPAND | gtk.FILL, 2, 2)
        video_box.pack_start(table, True)
        # Painting the drawing areas black
        for drawing_area in drawing_areas_list:
            drawing_area.connect('expose-event', PanelFactory.__draw_black_background, lines, columns)
            drawing_area.connect('configure_event', PanelFactory.__draw_black_background, lines, columns)
        return video_box
    
    @staticmethod
    def create_trackbar_panel(widgets_callback_dictionary, index):
        '''Create a trackbar panel, on which the trackbar is positioned
        @param widgets_callback_dictionary: a dictionary on which we'll insert the items that need a callback
        @return: gtk.HBox'''
        outer_box = gtk.HBox(True)
        widgets_callback_dictionary['Panel_Trackbar_' + index.__str__()] = outer_box
        adjustment = gtk.Adjustment(0.0, 0.0, 0.0, 0.0, 0.0, 0.0)
        trackbar = gtk.HScale(adjustment)
        trackbar.set_name("Callback_Trackbar_" + index.__str__())
        trackbar.set_draw_value(False)
        widgets_callback_dictionary[trackbar.get_name()] = trackbar
        outer_box.pack_start(trackbar, False)
        return outer_box

    @staticmethod
    def create_button_panel(widgets_callback_dictionary):
        '''Create a button panel, on which the buttons are displayed
        @param widgets_callback_dictionary: a dictionary on which we'll insert the items that need a callback
        @return gtk.HBox'''
        outer_box = gtk.HBox(False)
        # Empty space
        left_box = gtk.HBox(True)
        outer_box.pack_start(left_box, True, True)
        # The panel itself
        main_panel = gtk.HBox(False)
        prev_button = ButtonFactory.create_prev_button()
        main_panel.pack_start(prev_button, False)
        widgets_callback_dictionary[prev_button.get_name()] = prev_button
        
        rew_frame = gtk.Frame()
        rew_box = gtk.HBox(False)
        rew_button = ButtonFactory.create_rew_button()
        widgets_callback_dictionary[rew_button.get_name()] = rew_button
        rew_box.pack_start(rew_button, False)
        rew_entry = StandardComponentFactory.create_general_entry("Callback_Entry_Rewind", 3, "1", "Number of frames to be rewound")
        widgets_callback_dictionary[rew_entry.get_name()] = rew_entry
        rew_box.pack_start(rew_entry, False)
        rew_frame.add(rew_box)
        main_panel.pack_start(rew_frame, False)
        
        play_button = ButtonFactory.create_play_button()
        main_panel.pack_start(play_button, False)
        widgets_callback_dictionary[play_button.get_name()] = play_button
        
        pause_button = ButtonFactory.create_pause_button()
        main_panel.pack_start(pause_button, False)
        widgets_callback_dictionary[pause_button.get_name()] = pause_button
        pause_button.set_no_show_all(True)
        
        fwd_frame = gtk.Frame()
        fwd_box = gtk.HBox(False)
        fwd_entry = StandardComponentFactory.create_general_entry("Callback_Entry_FastForward", 3, "1", "Number of frames to be forwarded")
        widgets_callback_dictionary[fwd_entry.get_name()] = fwd_entry
        fwd_box.pack_start(fwd_entry, False)
        fwd_button = ButtonFactory.create_fwd_button()
        widgets_callback_dictionary[fwd_button.get_name()] = fwd_button
        fwd_box.pack_start(fwd_button, False)
        fwd_frame.add(fwd_box)
        main_panel.pack_start(fwd_frame, False)
        
#        next_frame = gtk.Frame()
#        next_box = gtk.HBox(False)
#        next_entry = StandardComponentFactory.create_general_entry("Callback_Entry_Next", 3, "", "Dummy")
#        widgets_callback_dictionary[next_entry.get_name()] = next_entry
#        next_box.pack_start(next_entry, False)
#        next_button = ButtonFactory.create_next_button()
#        widgets_callback_dictionary[next_button.get_name()] = next_button
#        next_box.pack_start(next_button, False)
#        next_frame.add(next_box)
#        main_panel.pack_start(next_frame, False)
        
        next_button = ButtonFactory.create_next_button()
        widgets_callback_dictionary[next_button.get_name()] = next_button
        main_panel.pack_start(next_button, False)
        outer_box.pack_start(main_panel, False)
        # Empty space
        right_box = gtk.HBox(True)
        outer_box.pack_start(right_box, True, True)
        return outer_box
    
    @staticmethod
    def create_statebar_panel(widgets_callback_dictionary, index):
        '''
        Creates a statebar panel, on which the state label and current frame position will show
        @param widgets_callback_dictionary: widgets_callback_dictionary
        '''
        outer_box = gtk.HBox(False)
        
        widgets_callback_dictionary['Panel_Statebar_' + str(index)] = outer_box
        
        video_label = StandardComponentFactory.create_general_label("Label_Video_Name_"  + str(index), "No video")
        outer_box.pack_start(video_label, False)
        widgets_callback_dictionary[video_label.get_name()] = video_label

        outer_box.pack_start(gtk.VSeparator(), False, False, 5)

        frame_label = gtk.Label("Frame: ")
        outer_box.pack_start(frame_label, False)

        current_frame_label = StandardComponentFactory.create_general_label("Label_Current_Frame_"  + str(index), "0")
        outer_box.pack_start(current_frame_label, False)
        widgets_callback_dictionary[current_frame_label.get_name()] = current_frame_label

        slash_label = gtk.Label(" / ")
        outer_box.pack_start(slash_label, False)

        total_frame_label = StandardComponentFactory.create_general_label("Label_Total_Frame_"  + str(index), "0")
        outer_box.pack_start(total_frame_label, False)
        widgets_callback_dictionary[total_frame_label.get_name()] = total_frame_label
        
        return outer_box
